# -*- coding: utf-8 -*-
"""
Created on Mon May 08 11:35:18 2017

@author: ppradeep
"""
## Import functions
import warnings
warnings.filterwarnings('ignore')

import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.font_manager import FontProperties 
from pandas.tools.plotting import scatter_matrix
import numpy as np

#path = 'Z:/Projects/PCB-NEQ'
#path = '/home/ppradeep/th034_home/Projects/PCB-NEQ'
#path = 'C:\Users\ppradeep\Desktop\PCB-NEQ'
## Import raw data from Simon paper
f0 = pd.read_csv('NEQ-allAssays.csv', index_col = 'CASRN')
del f0['Congener'] 
f0.columns=['Kodavanti PEB','Shain','Kodavanti Micro','Kodavanti Mito','Pessah RyR1 EC2x','Pessah RyR1', 'Stenberg Dop', 'Stenberg DAT', 'Wigestrand DAT']

#%%
   
for col in f0.columns:
    try:    
        f0.loc[(f0[col] == '>201'), col] = 'INACTIVE' 
        f0.loc[(f0[col] == '>100'), col] = 'INACTIVE' 
        f0.loc[(f0[col] == '>25'), col] = 'INACTIVE' 
        f0.loc[(f0[col] == 'NEO'), col] = 'INACTIVE' 
        f0.loc[(f0[col] == 'ASL'), col] = 'INACTIVE' 
        f0.loc[(f0[col] == 'INACTIVE'), col] = 'INACTIVE'
        f0.loc[(f0[col] == 'No Effect'), col] = 'INACTIVE'
    except:
        print(col)
        pass


#f0.replace('>201', 'INACTIVE', inplace = True)
#f0.replace('>100', 'INACTIVE', inplace = True)
#f0.replace('>25', 'INACTIVE', inplace = True)
#f0.replace('NEO', 'INACTIVE', inplace = True)
#f0.replace('ASL', 'INACTIVE', inplace = True)
#f0.replace('INACTIVE', 'INACTIVE', inplace = True)
#f0.replace('No Effect', 'INACTIVE', inplace = True)

#%%    
del f0['Pessah RyR1 EC2x']   
#del f0['Pessah RyR1 EC50']  
f0 = f0.dropna(axis = 0, how = 'all')

#%%
# PCB no. and CASRN mapping
pcb = pd.read_csv('PCBs.csv', index_col = 'CASRN')

#%%
##############################################################################################################
## Rescale data
# Method 1. Divide by median of non-zero values of the column and then divide by maximum value to scale from 0-1
# Method 2. Divide by value for congener 4 or 52 for which dota is available for all and then then divide by 
#           maximum value to scale from 0-1
##############################################################################################################


######################
##    Method 1      ##
######################
f0_normalized = f0.copy()
for col in f0_normalized.columns: 
    if col == 'Stenberg Dop' or col == 'Stenberg DAT':
        # Select index of all not null, active cells
        idx = f0_normalized[col].convert_objects(convert_numeric=True).dropna().index
        f0_normalized.loc[idx, col] = f0_normalized.loc[idx, col].apply(pd.to_numeric)
        # Scale:
        [f0_normalized[col].__setitem__([i], (100-f0_normalized.loc[i, col])/100) for i in idx]
    else:
        # Select index of all not null, active cells
        idx = f0_normalized[col].convert_objects(convert_numeric=True).dropna().index
        f0_normalized.loc[idx, col] = f0_normalized.loc[idx, col].apply(pd.to_numeric)
        # Scale: 1. Divide by median, 2. Divide by maximum
        col_med = f0_normalized.loc[idx, col].median()
        [f0_normalized[col].__setitem__([i], col_med/f0_normalized.loc[i, col]) for i in idx]
        col_max = f0_normalized.loc[idx, col].max()
        [f0_normalized[col].__setitem__([i], f0_normalized.loc[i, col]/col_max) for i in idx]    

#%%
# Set all the inactives to 0
f0_normalized.replace('INACTIVE', 0, inplace = True)
#f0_normalized.replace('No Effect', 0, inplace = True)
## Calculate the NEQ value as mean of all scaled columns
f0_normalized['NEQ'] = f0_normalized.mean(axis = 1, skipna = True)

#%%
## NEQs from Simon paper
df_Simon = pd.read_csv('NEQ.csv', index_col = 'CASRN')
df_Simon.rename(columns = {'NEQ': 'NEQ Simon et al'}, inplace=True)
df_Simon = df_Simon['NEQ Simon et al'].dropna(axis = 0)

#%%
f0_normalized['NEQ Simon et al'] = df_Simon
## Change all cells to numeric type
f0_normalized = f0_normalized.apply(pd.to_numeric)

#%%
# Save the re-derived NEQs to a CSV
pd.concat([pcb, f0_normalized], axis = 1).to_csv('NEQ_data.csv')

## Plot the data
df_neq = f0_normalized.copy()

#%%
### Distribution of NEQs 
### SCatter Plot: NEQ New data
#figure = plt.figure(dpi = 200)
#plt.scatter(x = df_neq.index, y = df_neq['NEQ'], marker='o')
#plt.xlim(1, 209)
#plt.ylim(-0.05,1.05)    
#plt.ylabel('NEQ', fontsize = 24)
#plt.xlabel('PCB Congener Number', fontsize = 24)
#plt.savefig(path+'/output/Final_NEQDistribution.png', bbox_inches='tight')
#
### Scatter plot: Simon Data
#figure = plt.figure(dpi = 200)
#plt.scatter(x = df_neq.index, y = df_neq['NEQ Simon et al'], marker='o')
#plt.xlim(1, 209)
#plt.ylim(-0.05,1.05)    
#plt.ylabel('NEQ', fontsize = 24)
#plt.xlabel('PCB Congener Number', fontsize = 24)
#plt.savefig(path+'/output/Simon_NEQDistribution.png', bbox_inches='tight')

## Distribution of NEQs - Histogram
## NEQ New and Simon data
fig = plt.figure(figsize=(6,10), dpi = 200)
df_neq.loc[:,['NEQ','NEQ Simon et al']].plot.hist(bins = 10, alpha = 0.5)
plt.xlabel('NEQ', fontsize = 18)
plt.ylabel('Frequency', fontsize = 18)
#plt.title('NEQ distribution in the dataset', fontsize = 16)
plt.show()
#plt.savefig(path+'/output/Final_NEQDistributionHist.png', bbox_inches='tight')

print(df_neq.loc[:,['NEQ','NEQ Simon et al']].corr())


df_final = df_neq.loc[(df_neq['NEQ']!=0)]
cols = df_final.columns.tolist()
# reorder columns
cols = cols[:8] + [cols[-1]] + [cols[-2]]
#cols = cols[:5] + [cols[-1]] + [cols[-2]]
# "commit" the reordering
df_final = df_final[cols]

#%%
### Plot all data vs each other with the pearson correlation coefficient
df_final.rename(columns = {'Shain': 'Shain PC-12 dop', 'Pessah RyR1': 'Pessah/Holland RyR1'}, inplace=True)
axes = scatter_matrix(df_final, alpha=0.8, figsize=(25, 25), color = 'r', s = 75, \
                      diagonal = 'hist', hist_kwds={'color':['green']})
axes[0,1].set_ylim(0,1)
axes[0,1].set_xlim(0,1)

corr = df_final.corr(method = 'pearson').as_matrix()
for i, j in zip(*plt.np.triu_indices_from(axes, k=1)):
    axes[i, j].annotate("Corr: %.2f" %corr[i,j], (0.7, 0.85), xycoords='axes fraction', ha='center', va='center', fontsize = 12)
# axis ticks
[plt.setp(item.yaxis.get_majorticklabels(), 'size', 9) for item in axes.ravel()]
[plt.setp(item.xaxis.get_majorticklabels(), 'size', 9) for item in axes.ravel()]
# labels
[plt.setp(item.yaxis.get_label(), 'size', 14) for item in axes.ravel()]
[plt.setp(item.xaxis.get_label(), 'size', 14) for item in axes.ravel()]
plt.show()
plt.savefig('NEQsCorrelation-All.png', bbox_inches='tight')

#%%
## Boxplot distribution of scaled data from all the experiments
df_final = df_final.drop(['NEQ', 'NEQ Simon et al'], axis = 1)

#%%
df_range = (df_final.max(axis=1, skipna = 1) - df_final.min(axis =1, skipna = 1))
#df_range = df_range[df_range != 0]
mean_range = df_range.mean()
fig = plt.figure(figsize=(10,8), dpi = 200)
ax = df_range.hist(bins = 5, alpha = 0.5)
ax.grid('off')
ax.text(0.70, 31, 'Mean = %.2f' % mean_range, fontsize = 20)
plt.xlabel('REP range', fontsize = 24)
plt.ylabel('Frequency', fontsize = 24) 
plt.show()
plt.savefig('Final_REPDistributionHist.png', bbox_inches='tight')

#%%
df_final.transpose().plot.box(return_type='axes', figsize=(20,12), rot = 90)
plt.scatter(x = range(1, len(df_neq.loc[df_final.index, 'NEQ'])+1), y = df_neq.loc[df_final.index, 'NEQ'], \
             s = 50, alpha=0.5, marker='o', c = 'red', label = 'NEF')
#plt.scatter(x = range(1, len(df_neq.loc[df_final.index, 'NEQ Simon et al'])+1), y = df_neq.loc[df_final.index, 'NEQ Simon et al'], \
#             s = 50, alpha=0.5, marker='o', c = 'green', label = 'NEQ Simon et al')
plt.ylim(-0.05,1.05)  

xticklabel = pcb.ix[df_final.index]['Congener'].tolist()

fontP = FontProperties()
fontP.set_size('20') 
plt.legend(prop = fontP, scatterpoints = 1, loc = [0.05, 0.9])
plt.ylabel('Neurotoxic Relative Potency', fontsize = 24)
plt.xlabel('PCB Congener Number', fontsize = 24)

plt.xticks(range(1,len(xticklabel)+1), xticklabel)

plt.savefig('OriginalRescaledData-BoxPlots.png', bbox_inches='tight')

