# -*- coding: utf-8 -*-
"""
Created on Tue Jan 17 15:27:39 2017

@author: ppradeep
"""

## Import functions
import warnings
warnings.filterwarnings('ignore')

import pandas as pd
import matplotlib.pyplot as plt
import numpy as np

from sklearn import cross_validation
import sklearn.metrics as sm

from sklearn import linear_model
from sklearn import svm, tree
from sklearn.neighbors import KNeighborsRegressor
from sklearn.ensemble import RandomForestRegressor

from sklearn.grid_search import GridSearchCV

# Set the path
path = 'Z:/Projects/PCB-NEQ'
#path = 'C:\Users\ppradeep\Desktop\PCB-NEQ'
#path = '/home/ppradeep/th034_home/Projects/PCB-NEQ'
    
# Read the input file with Cl substitution data
f0 = pd.read_csv(path+'/data/NEQ.csv', index_col = 'CASRN')
features = f0.filter(regex = 'R-Group')

# Read derived NEQ values for training the model
neq = pd.read_csv(path+'/output/NEQ_data.csv', index_col = 'CASRN')['NEQ'].dropna()

## MOE descriptors
#moe = pd.read_csv(path+'/data/pcb_moe3d.csv', index_col = 'CASRN').ix[:,['AM1_dipole','AM1_IP', 'vol']]
#moe = pd.read_csv(path+'/data/pcb_moe.csv', index_col = 'CASRN')

# PCB no. and CASRN mapping
pcb = pd.read_csv(path+'/data/PCBs.csv', index_col = 'CASRN')
#%%
# Construct Fingerprint 1
fingerprint1 = pd.concat([features, features.sum(axis = 1)], axis = 1)

# Construct Fingerprint 2
f1 = f0['R-Group #6'] + f0['R-Group #7'] + f0['R-Group #8'] + f0['R-Group #9']
f2 = f0['R-Group #1'] + f0['R-Group #2'] + f0['R-Group #4'] + f0['R-Group #10']
f3 = f0['R-Group #3'] + f0['R-Group #5'] 
fingerprint2 = pd.concat([f1, f2, f3, f1+f2+f3], axis = 1)

# Construct Fingerprint 3
f1 = f0['R-Group #6'] + f0['R-Group #9'] #Ortho ring2
f2 = f0['R-Group #7'] + f0['R-Group #8'] #Ortho ring 1
f3 = f0['R-Group #1'] + f0['R-Group #4'] #Meta ring 1
f4 = f0['R-Group #2'] + f0['R-Group #10'] #Meta ring 2
f5 = f0['R-Group #3'] + f0['R-Group #5']  #Para
fingerprint3 = pd.concat([f1, f2, f3, f4, f5, f1+f2+f3+f4+f5], axis = 1)


#%%
#[neq.__setitem__(int(idx), f0.loc[int(idx), 'NEQ New']) for idx in f0[f0['NEQ New'].notnull()].index] #
#### Set values for null data coulmns = valuesusing empirical model from the Simon data 
#[neq.__setitem__(int(idx), df_Simon.loc[int(idx), 'NEQ Simon et al']) for idx in df_Simon[df_Simon['NEQ Simon et al'].notnull()].index if np.isnan(f0.loc[int(idx), 'NEQ New'])] #

# Set the data for modeling
rmse = {}; r2 = {}; var = {}
dt_params = {}; svm_params = {}; knn_params = {}; rf_params = {}
for fingerprint in [1,2,3]: # Select fingerprint
    if fingerprint == 1:
        X = fingerprint1.loc[neq.index, :]
    if fingerprint == 2:
        X = fingerprint2.loc[neq.index, :]
    if fingerprint == 3:
        X = fingerprint3.loc[neq.index, :]
    Y = neq
    
    ## Evaluate the hyperparameters for all models based on n_fold fold CV
    n_fold = 5

#    ## DT
#    parameters =  {'max_depth': [5,6,7], 'max_features': ['sqrt', 'auto'], 'presort':['true']} #max_features = "sqrt",  min_samples_leaf = 2, 
#    clf = tree.DecisionTreeRegressor()
#    grid_search = GridSearchCV(clf, cv = n_fold, param_grid = parameters)
#    grid_search.fit(X, Y)
#    dt_params[fingerprint] = grid_search.best_params_
    ## SVM
    parameters = {'kernel':['linear', 'rbf'], 'C':[0.1, 1, 10], 'gamma':[0.1, 1], 'epsilon': [0, 0.1]}
    clf = svm.SVR()
    grid_search = GridSearchCV(clf, cv = n_fold, param_grid = parameters)
    grid_search.fit(X, Y)
    svm_params[fingerprint] = grid_search.best_params_
    # KNN
    parameters = {'weights':['uniform', 'distance'], 'n_neighbors':[1,2,3], 'algorithm': ['auto', 'ball_tree', 'kd_tree', 'brute']}
    clf = KNeighborsRegressor()
    grid_search = GridSearchCV(clf, cv = n_fold, param_grid = parameters)
    grid_search.fit(X, Y)
    knn_params[fingerprint] = grid_search.best_params_
    ## Random Forest
    parameters = {"n_estimators": [25, 50], "max_features": ['sqrt', 'auto'], 'random_state':[5]}
    clf = RandomForestRegressor()
    grid_search = GridSearchCV(clf, cv = n_fold, param_grid = parameters)
    grid_search.fit(X, Y)
    rf_params[fingerprint] = grid_search.best_params_

    ## Train the model and calculate errors using n_fold cross validation
    num_instances = len(X)
    seed = 8
    
    kfold = cross_validation.KFold(n=num_instances, n_folds=n_fold, random_state=seed)
    classifiers = [ 
        linear_model.LinearRegression(),
        #tree.DecisionTreeRegressor(**dt_params[fingerprint]),
        svm.SVR(**svm_params[fingerprint]),
        KNeighborsRegressor(**knn_params[fingerprint]),
        RandomForestRegressor(**rf_params[fingerprint]) # 
        ]
    
    for clf in classifiers:
        Y_predicted = cross_validation.cross_val_predict(clf, X = X, y = Y, cv = n_fold)
        rms = np.sqrt(sm.mean_squared_error(Y, Y_predicted))
        rmse.setdefault(fingerprint,[]).append(np.sqrt(sm.mean_squared_error(Y, Y_predicted)))
        r_2 = sm.r2_score(Y, Y_predicted)
        r2.setdefault(fingerprint, []).append(r_2)
        
        fig, ax = plt.subplots(figsize=(10,8), dpi = 200)
        #fig = plt.figure(figsize=(10,8), dpi = 200)
        #ax2 = plt.scatter(x = Y, y = Y_predicted, s = 25, alpha = 0.75, marker='o', c = 'red')
        ax.scatter(x = Y, y = Y_predicted, s = 25, alpha = 0.75, marker='o', c = 'red')
        plt.plot([0,1], [0,1], 'k')
        plt.plot([0.1,1.1],[0,1],'g', linestyle = '--')
        plt.plot([-0.1, 0.9],[0,1],'g', label = '$\pm$ 0.1 error interval', linestyle = '--')
        plt.legend(loc='lower right', numpoints = 1, scatterpoints = 1, fontsize = 16)
        plt.xlim([0,1])
        plt.ylim([0,1])  
        plt.xlabel('Derived NEQ', fontsize = 18)
        plt.ylabel('Predicted NEQ', fontsize = 18)
        ax.annotate(r'$RMSE: %.2f$' % rms, [0.05, .95], fontsize = 18)
        ax.annotate(r'$R^{2}: %.2f$' % r_2, [0.05, .85], fontsize = 18)
#        ax.text(0.05, .9, r'$RMSE: %.2f$' % rms, fontsize = 16)
#        ax.text(0.05, .8, r'$R^{2}: %.2f$' % r_2, fontsize = 16)
        #plt.savefig(path+'/output/TruevsPred_Int_%s%d.png' %(str(clf).split('(')[0], fingerprint), bbox_inches='tight')

#%%
## Average Training and Validation rmse
fig = plt.figure(figsize=(8,6), dpi = 200) #, 
ax = fig.add_subplot(111)

x = range(4)
x_label = ['LR','SVR', 'kNN', 'RF'] #'Decision \n Tree',  , 'Consensus'
color = ['red', 'blue', 'green']

for fingerprint in [1,2,3]:
    plt.plot(x, rmse[fingerprint], '--o', color = color[fingerprint-1], label = 'Fingerprint %d' %fingerprint, markersize=10)

#ax.set_xlabel('Decision Tree Attributes', fontsize = 14)
ax.set_ylabel('RMSE', fontsize = 18)
#plt.title('Performance Metrics (Fingerprint 3)', fontsize = 14)
plt.xticks(x, x_label, rotation = 0, fontsize = 16)
plt.xlim([-0.05, 3.05])
plt.ylim([0.1, 0.35])
plt.legend(loc = 'lower right',  fontsize = 18, numpoints = 1) 
plt.savefig(path+'/output/PredictionErrors.png', bbox_inches='tight')

#%%
## Average Training and Validation R2
fig = plt.figure(figsize=(8,6), dpi = 200) #, 
ax = fig.add_subplot(111)

for fingerprint in [1,2,3]:
    plt.plot(x, r2[fingerprint], '--o', color = color[fingerprint-1], label = 'Fingerprint %d' %fingerprint, markersize=10)

#ax.set_xlabel('Decision Tree Attributes', fontsize = 14)
ax.set_ylabel('$R^{2}$', fontsize = 18)
#plt.title('Performance Metrics (Fingerprint 3)', fontsize = 14)
plt.xticks(x, x_label, rotation = 0, fontsize = 16)
plt.xlim([-0.05, 3.05])
plt.ylim([-1, 0.1])
plt.legend(loc = 'lower right', fontsize = 18, numpoints = 1) 
plt.savefig(path+'/output/PredictionR2.png', bbox_inches='tight')

#%%
### Train on complete data and predict for unknowns
## Set the training and test data

## From above select best fingerprint
X = fingerprint3.loc[neq.index, :]
Y = neq

# Predict unknown
X_predict = fingerprint3.drop(neq.index)

# Predict all
X_predict = fingerprint3

fingerprint = 3
clf1 = linear_model.LinearRegression()  
clf2 = svm.SVR(**svm_params[fingerprint])
clf3 = KNeighborsRegressor(**knn_params[fingerprint])
clf4 = RandomForestRegressor(**rf_params[fingerprint])
#clf5 = tree.DecisionTreeRegressor(**dt_params[fingerprint])

                
clf1 = clf1.fit(X, Y)
clf2 = clf2.fit(X, Y)
clf3 = clf3.fit(X, Y)
clf4 = clf4.fit(X, Y)
#clf5 = clf5.fit(X, Y)

r2_1 = clf1.score(X,Y)
r2_2 = clf2.score(X,Y)
r2_3 = clf3.score(X,Y)
r2_4 = clf4.score(X,Y)
#r2_5 = clf5.score(X,Y)

Y_predict1 = pd.DataFrame({'Predicted NEQ (Linear Regression)': clf1.predict(X_predict)})
Y_predict1.index = X_predict.index

Y_predict2 = pd.DataFrame({'Predicted NEQ (SVR)': clf2.predict(X_predict)})
Y_predict2.index = X_predict.index

Y_predict3 = pd.DataFrame({'Predicted NEQ (kNN)': clf3.predict(X_predict)})
Y_predict3.index = X_predict.index

Y_predict4 = pd.DataFrame({'Predicted NEQ (RF)': clf4.predict(X_predict)})
Y_predict4.index = X_predict.index

#Y_predict5 = pd.DataFrame({'Predicted NEQ (DT)': clf5.predict(X_predict)})
#Y_predict5.index = X_predict.index

NEQ_outfile = pd.concat([pcb, neq, Y_predict1, Y_predict2, Y_predict3, Y_predict4], axis = 1)
NEQ_outfile.to_excel(path+'/output/PredictedNEQ.xls')

#%%
names = fingerprint3.columns
print(sorted(zip(map(lambda x: round(x, 4), clf4.feature_importances_), names), 
             reverse=True))

xlabels = ['Ortho \nRing 2', 'Ortho \nRing 1', 'Meta \nRing 1', 'Meta \nRing 2', 'Para', 'Total']
importances = clf4.feature_importances_   
std = np.std([tree.feature_importances_ for tree in clf4.estimators_], axis=0)
indices = np.argsort(importances)[::-1]   
      
# Plot the feature importances of the forest
plt.figure(figsize=(10,8), dpi = 200)
plt.bar(range(X.shape[1]), importances[indices], color="r", yerr=std[indices], align="center", alpha = 0.75)
#plt.xticks(range(X.shape[1]), indices)
plt.xticks(range(X.shape[1]), [xlabels[i] for i in indices], rotation = 90, fontsize = 16)       
plt.xlim([-1, X.shape[1]])
plt.xlabel('Number of Cl substitutions', fontsize = 18, labelpad = 20)
plt.ylabel('Descriptor Importance', fontsize = 18, labelpad = 20)
plt.savefig(path+'/output/RF_FeatureImp.png', bbox_inches='tight')

##%%
### True vs Predicted Plots
#fig = plt.figure(figsize=(10,8), dpi = 200)
#ax1 = NEQ_outfile.plot.scatter(x = 'NEQ', y = 'Predicted NEQ (Linear Regression)', s = 25, alpha = 0.75, marker='o', c = 'red')
#plt.plot([0,1], [0,1], 'k')
#plt.plot([0.1,1.1],[0,1],'g', linestyle = '--')
#plt.plot([-0.1, 0.9],[0,1],'g', label = '$\pm$ 0.1 error interval', linestyle = '--')
#plt.legend(loc='lower right', numpoints = 1, scatterpoints = 1, fontsize = 16)
#plt.xlim([0,1])
#plt.ylim([0,1])  
#plt.xlabel('Derived NEQ', fontsize = 18)
#plt.ylabel('Predicted NEQ', fontsize = 18)
#ax1.text(0.05, .9, r'$RMSE: %.2f$' % rmse[3][0], fontsize = 18)
#ax1.text(0.05, .8, r'$R^{2}: %.2f$' % r2[3][0], fontsize = 18)
#plt.savefig(path+'/output/TruevsPred_LR.png', bbox_inches='tight')
#
#
#fig = plt.figure(figsize=(10,8), dpi = 200)
#ax2 = NEQ_outfile.plot.scatter(x = 'NEQ', y = 'Predicted NEQ (SVR)', s = 25, alpha = 0.75, marker='o', c = 'red')
#plt.plot([0,1], [0,1], 'k')
#plt.plot([0.1,1.1],[0,1],'g', linestyle = '--')
#plt.plot([-0.1, 0.9],[0,1],'g', label = '$\pm$ 0.1 error interval', linestyle = '--')
#plt.legend(loc='lower right', numpoints = 1, scatterpoints = 1, fontsize = 16)
#plt.xlim([0,1])
#plt.ylim([0,1])  
#plt.xlabel('Derived NEQ', fontsize = 18)
#plt.ylabel('Predicted NEQ', fontsize = 18)
#ax2.text(0.05, .9, r'$RMSE: %.2f$' % rmse[3][1], fontsize = 18)
#ax2.text(0.05, .8, r'$R^{2}: %.2f$' % r2[3][1], fontsize = 18)
#plt.savefig(path+'/output/TruevsPred_SVR.png', bbox_inches='tight')
#
#fig = plt.figure(figsize=(10,8), dpi = 200)
#ax3 = NEQ_outfile.plot.scatter(x = 'NEQ', y = 'Predicted NEQ (kNN)', s = 30, alpha = 0.75, marker='o', c = 'red')
#plt.plot([0,1], [0,1], 'k')
#plt.plot([0.1,1.1],[0,1],'g', linestyle = '--')
#plt.plot([-0.1, 0.9],[0,1],'g', label = '$\pm$ 0.1 error interval', linestyle = '--')
#plt.legend(loc='lower right', numpoints = 1, scatterpoints = 1, fontsize = 16)
#plt.xlim([0,1])
#plt.ylim([0,1])  
#plt.xlabel('Derived NEQ', fontsize = 18)
#plt.ylabel('Predicted NEQ', fontsize = 18)
#ax3.text(0.05, .9, r'$RMSE: %.2f$' % rmse[3][2], fontsize = 18)
#ax3.text(0.05, .8, r'$R^{2}: %.2f$' % r2[3][2], fontsize = 18)
#plt.savefig(path+'/output/TruevsPred_kNN.png', bbox_inches='tight')
#
#fig = plt.figure(figsize=(10,8), dpi = 200)
#ax4 = NEQ_outfile.plot.scatter(x = 'NEQ', y = 'Predicted NEQ (RF)', s = 25, alpha = 0.75, marker='o', c = 'red')
#plt.plot([0,1], [0,1], 'k')
#plt.plot([0.1,1.1],[0,1],'g', linestyle = '--')
#plt.plot([-0.1, 0.9],[0,1],'g', label = '$\pm$ 0.1 error interval', linestyle = '--')
#plt.legend(loc='lower right', numpoints = 1, scatterpoints = 1, fontsize = 16)
#plt.xlim([0,1])
#plt.ylim([0,1])  
#plt.xlabel('Derived NEQ', fontsize = 18)
#plt.ylabel('Predicted NEQ', fontsize = 18)
#ax4.text(0.05, .9, r'$RMSE: %.2f$' % rmse[3][3], fontsize = 18)
#ax4.text(0.05, .8, r'$R^{2}: %.2f$' % r2[3][3], fontsize = 18)
#plt.savefig(path+'/output/TruevsPred_RF.png', bbox_inches='tight')
